from datetime import datetime, timedelta
from io import StringIO

import requests
import pandas as pd
from pyquery import PyQuery as pq

from . import __fund_base as fund_base


class Nikko(fund_base.FundBase):

    def __init__(self, id, name=None):
        super().__init__(id, name)

    def csv_url(self):
        return "https://www.nikkoam.com/products/detail/%s/data/?format=xls" % self.id

    def detail_url(self):
        return "https://www.nikkoam.com/products/detail/%s" % self.id

    def get(self):
        url = self.csv_url()
        response = requests.get(url)
        if response.status_code / 100 == 2:
            response.encoding = "utf-8"
            # print(response.text)
            def my_parser(date): return pd.datetime.strptime(date, "%Y/%m/%d")
            return pd.read_csv(StringIO(response.text),
                               names=('Date', 'Price',
                                      'Change', 'Distribution', 'Total assets', 'Gross'),
                               header=1,
                               date_parser=my_parser,
                               index_col=0)
        else:
            raise "error"

    @classmethod
    def get_list(cls):
        html = pq(url="https://www.nikkoam.com/products/hotnews")
        contents = html(".com_hotnews")
        table = contents("#tableWithFloatingHeader")
        link_list = table(".fund_name")("a")
        link_name = [[pq(i).attr("href"), pq(i).text().replace("\n", " ")]
                     for i in link_list]
        id_name = [[i[0].split("/")[-2], i[1]] for i in link_name]
        return [Nikko(e[0], e[1]) for e in id_name]



from datetime import datetime, timedelta
from io import StringIO

import requests
import pandas as pd

from . import __fund_base as fund_base


class Morningstar(fund_base.FundBase):

    def __init__(self, id, name=None):
        super().__init__(id, name)

    def csv_url(self):
        return "http://www.morningstar.co.jp/FundData/DownloadStdYmd.do?fnc=" + self.id

    def detail_url(self):
        return "http://www.morningstar.co.jp/FundData/SnapShot.do?fnc=" + self.id

    def get(self):
        url = self.csv_url()
        tomorrow = datetime.now() + timedelta(days=1)
        data = {"selectStdYearFrom": "1990",
                "selectStdMonthFrom": "1",
                "selectStdDayFrom": "1",
                "selectStdYearTo": str(tomorrow.year),
                "selectStdMonthTo": str(tomorrow.month),
                "selectStdDayTo": str(tomorrow.day),
                "base": "0"}
        response = requests.post(url, data=data)
        if response.status_code / 100 == 2:
            response.encoding = "sjis"
            def my_parser(date): return pd.datetime.strptime(date, "%Y%m%d")
            return pd.read_csv(StringIO(response.text),
                               names=('Date', 'Price'),
                               header=0,
                               date_parser=my_parser,
                               index_col=0)
        else:
            raise "error"



class FundBase:

    def __init__(self, id, name=None):
        self.__id = id
        if name is None:
            self.__name = id
        else:
            self.__name = name

    def get(self):
        return None

    def csv_url(self):
        return None

    def detail_url(self):
        return None

    @property
    def id(self):
        return self.__id

    @property
    def name(self):
        return self.__name

    def __str__(self):
        return "%s, %s" % (self.id, self.name)



from datetime import datetime, timedelta
from io import StringIO

import requests
import pandas as pd
from pyquery import PyQuery as pq

from . import __fund_base as fund_base


class EMaxis(fund_base.FundBase):

    def __init__(self, id, name=None):
        super().__init__(id, name)

    def csv_url(self):
        return "https://emaxis.jp/content/csv/fundCsv.php?fund_cd=%s" % self.id

    def detail_url(self):
        return "https://emaxis.jp/fund/%s.html" % self.id

    def get(self):
        url = self.csv_url()
        response = requests.get(url)
        if response.status_code / 100 == 2:
            response.encoding = "sjis"
            # print(response.text)
            def my_parser(date): return pd.datetime.strptime(date, "%Y/%m/%d")
            return pd.read_csv(StringIO(response.text),
                               names=('Date', 'Price',
                                      'Gross', 'Distribution', 'Total assets'),
                               header=1,
                               date_parser=my_parser,
                               index_col=0)
        else:
            raise "error"

    @classmethod
    def get_list(cls):
        html = pq(url="https://emaxis.jp/fund/index.html")
        contents = html(".contentswrap")
        link_list = contents("a")
        id_name = [[pq(i).attr("href").replace(".html", ""), pq(i).text().replace("\n", " ")]
                   for i in link_list]
        return [EMaxis(e[0], e[1]) for e in id_name]



from jpfund.__morningstar import Morningstar
from jpfund.__emaxis import EMaxis
from jpfund.__nikko import Nikko

__copyright__ = 'Copyright (C) 2018 kunigaku'
__version__ = '0.0.1'
__license__ = 'MIT'
__author__ = 'kunigaku'
__author_email__ = 'kunigaku@gmail.com'
__url__ = 'https://github.com/kunigaku/jpfund'

__all__ = []
